# -*-python-*-
# $Id: textgenerator.py,v 1.4 2006/04/02 20:13:55 asc Exp $

__package__    = "erdfg"
__version__    = "0.3"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/erdfg/"
__cvsversion__ = "$Revision: 1.4 $"
__date__       = "$Date: 2006/04/02 20:13:55 $"
__copyright__  = "Copyright (c) 2006 Aaron Straup Cope. Perl Artistic License."

from erdfg.generator.base import base
import textwrap
import codecs

class textgenerator (base) :
            
    def write_stage (self, stage, fh) :
        fh.write("-" * 72 + "\n")
        fh.write((" " * (71 - len(stage['title']))) + " " + stage['title'] + "\n")
        fh.write("-" * 72 + "\n")

        self.write_ingredients(stage['ingredients'], fh)
        self.write_directions(stage['directions'], fh)        

    #
    #
    #
    
    def write_ingredients (self, list, fh) :
        self.write_label("ingredients", fh)

        for ing in list :
            if ing.has_key('fallback') :
                self.write_xinclude(ing, fh)
            else :            
                self.write_ingredient(ing, fh)

        fh.write("\n")

    #
    #
    #

    def write_directions (self, list, fh) :
        self.write_label("directions", fh)

        for entry in list :
            if entry.has_key('fallback') :
                self.write_xinclude(entry, fh)
            else :
                self.write_content(entry, fh)

        fh.write("\n\n")

    #
    #
    #

    def write_notes (self, list, fh) :
        self.write_label("notes", fh)
        
        for entry in list :
            self.write_content(entry, fh)

        fh.write("\n\n")
    
    #
    #
    #

    def write_history (self, list, fh) :
        self.write_label("history", fh)

        for entry in list :
            self.write_content(entry, fh)

        fh.write("\n\n")

    #
    #
    #

    def write_yield (self, list, fh) :
        self.write_label("yield", fh)

        self.write_kv_pairs(list, fh)

        fh.write("\n")
        
    #
    #
    #
    
    def write_kv_pairs (self, dict, fh) :
        for k, v in dict.items() :
            fh.write(self.prepare("%s : %s" % (k.capitalize(), v)))
            fh.write("\n")
            
    #
    #
    #

    def write_time (self, times, fh) :
        self.write_label("time", fh)
        self.write_kv_pairs(times, fh)
        fh.write("\n")

    #
    #
    #

    def write_source (self, source, fh) :
        self.write_label("source", fh)
        self.write_kv_pairs(source, fh)
        fh.write("\n")
        
    #
    #
    #
    
    def write_ingredient (self, ing, fh) :
        fmt = "%s "
        str = ""
        
        for label in ('amount', 'measure', 'foodstuff', 'detail') :
            if ing.has_key(label) and not ing[label]==None :
                
                if label=='detail' :
                    fmt = u"(%s)"
                    
                str = str + fmt % ing[label]

        fh.write(self.prepare(str, u"* "))
        fh.write("\n")

    #
    #
    #

    def write_xinclude (self, xinclude, fh) :
        fh.write(self.prepare("Refer to : %s" % xinclude['fallback']))
        fh.write("\n")
        
    #
    #
    #
 
    def write_content (self, entry, fh) :
        for label in ('content', 'author', 'date') :
            if entry.has_key(label) and not entry[label]==None :

                if label == 'author' or label == 'date' :
                    fh.write("%s : " % label)
                             
                fh.write(self.prepare(entry[label]))
                fh.write("\n")
                
    #
    #
    #
    
    def write_label (self, label, fh) :
        fh.write(label.upper())
        fh.write(u"\n\n")

    #
    #
    #
    
    def prepare (self, txt, prefix="") :

        t = textwrap.TextWrapper()
        t.width = 72
        t.initial_indent = prefix

        newlines = "\n\n"
        parts    = []
                
        for p in txt.split(newlines) :
            parts.append(t.fill(p.strip()))

        return newlines.join(parts).encode("utf8")
